/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.UIJob;

import de.bsvrz.buv.plugin.konfigass.konsoleAusgabeUpdateListener.IKonsoleAusgabeUpdateListener;
import de.bsvrz.buv.plugin.konfigass.konsoleAusgabeUpdateListener.KonsoleAusgabeUpdateEvent;
import de.bsvrz.buv.plugin.konfigass.konsoleAusgabeUpdateListener.KonsoleAusgabeUpdateListener;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Speichert bisher empfangene Betriebsmeldungen. <br>
 * Hinweis: Diese Klasse ist als Singleton ausgelegt, damit auch zu einem spteren Zeitpunkt, nach dem schlieen des
 * Sicht {@link MeldungsanzeigeDialog}, alle bereits erzeugten Meldungen wieder angezeigt werden knnen.
 * 
 * @version $Revision: 1.16 $ / $Date: 2011/01/17 10:06:18 $ / ($Author: Sans $)
 * @author HofmannJ
 * 
 */
public class MeldungenSpeicher implements IKonsoleAusgabeUpdateListener
{

  /**
   * Speichert die einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   */
  private static MeldungenSpeicher _instanz = null;

  /**
   * Gibt die Maximale Anzahl von Betriebsmeldungen an.
   */
  private static int _maximaleAnzahlMultiMeldungen = 1000;

  /**
   * Speichert die empfangenen Betriebsmeldungen und hlt diese vorrtig.
   */
  private Map<IStatus, Object[]> _lMultiMeldungen = null;

  private MultiStatus _aktuelleMeldung;

  private long _aktuellerZeitstempel;

  private PrintStream errBackup;

  private PrintStream outBackup;

  private KonsoleAusgabeStream _kos;

  private Debug _debug = Debug.getLogger();

  private boolean _updateErhalten;

  private boolean canFinfish;
  
  private String _konfiguration = "";

  /**
   * Liefert die einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   * 
   * @return Einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   */
  public static MeldungenSpeicher getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new MeldungenSpeicher(_maximaleAnzahlMultiMeldungen);
    }
    return _instanz;
  }

  /**
   * Liefert die einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   * 
   * @param maxAnzahlMultiMeldungen
   *          Setzt die maximale Anzahl von Betriebsmeldungen, die gespeichert werden. Wird nur bei der ersten
   *          Initialisierung der Singletonklasse gesetzt und ist im Nachhinein (z.B. beim zweiten Aufruf dieser
   *          Methode) nicht vernderbar. Defaultwert == 1000.
   * 
   * @return Einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   */
  public static MeldungenSpeicher getInstanz(int maxAnzahlMultiMeldungen)
  {
    if (_instanz == null)
    {
      _instanz = new MeldungenSpeicher(maxAnzahlMultiMeldungen);
    }
    return _instanz;
  }

  /**
   * Privater, nicht ffentlicher Konstruktor dieser Singletonklasse.
   */
  private MeldungenSpeicher()
  {
    init();
  }

  /**
   * Privater, nicht ffentlicher Konstruktor dieser Singletonklasse.
   * 
   * @param maxAnzahlMultiMeldungen
   *          Setzt die maximale Anzahl von Meldungen, die gespeichert werden.
   */
  private MeldungenSpeicher(int maxAnzahlMultiMeldungen)
  {
    _maximaleAnzahlMultiMeldungen = maxAnzahlMultiMeldungen;
    init();
  }

  /**
   * Fhrt die Initialisierung durch.
   * 
   */
  private void init()
  {
    _lMultiMeldungen = new HashMap<IStatus, Object[]>();

    this.errBackup = System.err;
    this.outBackup = System.out;

    _kos = new KonsoleAusgabeStream(this.errBackup);
    KonsoleAusgabeUpdateListener.getInstanz().addKonsoleAusgabeUpdateListener(this);

    System.setErr(new PrintStream(_kos, true));
    Debug.reinstallConsoleHandler();
    _debug.setLoggerLevel(StdOutErrLevel.FINEST);
  }

  /**
   * Fgt der Liste der Betriebsmeldungen eine neue Betriebsmeldung am Anfang (Index 0) hinzu. Nachfolgende, "ltere"
   * Elemente werden um eine Position nach hinten verschoben. <br>
   * bersteigt beim Hinzufgen einer Betriebsmeldung die Anzahl der Betriebsmeldungen die maximale Anzahl von
   * Multimeldungen, so wird die jeweils lteste (am frhsten hinzugefgte) Multimeldung gelscht.
   * 
   * @param multiMeldung
   *          Multimeldung (Instanz der Klasse {@link Meldung}), die hinzugefgt werden soll.
   */
  public void hinzufuegenMultimeldung(IStatus multiMeldung, Long timeStamp, boolean ausgabeErwartet)
  {

//    System.out.println("MeldungenSpeicher.hinzufuegenMultimeldungNeu(): " + ausgabeErwartet);

    _aktuelleMeldung = (MultiStatus)multiMeldung;
    _aktuellerZeitstempel = timeStamp;

    if (!ausgabeErwartet)
    {
      hinzufuegenMultimeldung(multiMeldung, timeStamp);
    }
    else
    {

      if (_aktuelleMeldung != null)
      {        
          MultiStatus multiStatus = (MultiStatus)multiMeldung;          
          _aktuelleMeldung = multiStatus;
          
          
          hinzufuegenMultimeldung(_aktuelleMeldung, timeStamp);
//          hinzufuegenMultimeldung();
          canFinfish = false;
      }
    }

  }

  private void hinzufuegenMultimeldung(final IStatus multiMeldung, Long timeStamp)
  {
    
    final String konfiguration = getKonfiguration();
    
    if (!_lMultiMeldungen.containsKey(multiMeldung)){
      Object[] objs = new Object[2];
      objs[0] = timeStamp;
      objs[1] = konfiguration;
      _lMultiMeldungen.put(multiMeldung, objs);
    }

    
    UIJob uiJob = new UIJob("") {
      
      @Override
      public IStatus runInUIThread(IProgressMonitor monitor)
      {
        // TODO Auto-generated method stub
        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        if (dw != null)
        {
          IWorkbenchPage page = dw.getActivePage();
          if (page != null)
          {
            IViewReference ref = page.findViewReference(ErrorView.ID);

            if (ref != null)
            {
              ErrorView view = (ErrorView)ref.getPart(false);

              if (view != null)
              {
                Object[] objs = _lMultiMeldungen.get(multiMeldung);
                Long l = (Long)objs[0];
                String s = (String)objs[1];
                view.getProtokoll().hinzufuegenMeldung(multiMeldung, l, s);
                // _kos.setStatus(null);
              }
            }
          }
        }        
        return Status.OK_STATUS;
      }
    };  
    uiJob.setUser(false);
    uiJob.setSystem(false);
    uiJob.schedule();

    /*
     * Prfen, ob die Anzahl der Multimeldungen die Anzahl der maximal zulssigen Multimeldungen bersteigt.
     */
    if (_lMultiMeldungen.size() > _maximaleAnzahlMultiMeldungen)
    {
      removeAeltesteMultimeldung();
    }
  }
  
//  public void hinzufuegenMultimeldung()
//  {
//    final String konfiguration = "XYZ";
//    
//    if (!_lMultiMeldungen.containsKey(_aktuelleMeldung))
//      _lMultiMeldungen.put(_aktuelleMeldung, _aktuellerZeitstempel);
//    
//    Display.getDefault().asyncExec(new Runnable() {
//      
//      @Override
//      public void run()
//      {
//        // TODO Auto-generated method stub
//        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
//        if (dw != null)
//        {
//          IWorkbenchPage page = dw.getActivePage();
//          if (page != null)
//          {
//            IViewReference ref = page.findViewReference(ErrorView.ID);
//            
//            if (ref != null)
//            {
//              ErrorView view = (ErrorView)ref.getPart(false);
//              
//              if (view != null)
//              {
//                Long l = _lMultiMeldungen.get(_lMultiMeldungen);
//                view.getProtokoll().hinzufuegenMeldung(_aktuelleMeldung, l, konfiguration);
//                // _kos.setStatus(null);
//              }
//            }
//          }
//        }
//      }
//    });
//    
//    /*
//     * Prfen, ob die Anzahl der Multimeldungen die Anzahl der maximal zulssigen Multimeldungen bersteigt.
//     */
//    if (_lMultiMeldungen.size() > _maximaleAnzahlMultiMeldungen)
//    {
//      removeAeltesteMultimeldung();
//    }
//  }

  public Map<IStatus, Object[]> getlMultiMeldungen()
  {
    return _lMultiMeldungen;
  }

  /**
   * Entfernt die lteste Multimeldung in der Liste der Betriebstmeldungen.
   * 
   */
  private void removeAeltesteMultimeldung()
  {
    int letzteElementPosition = _lMultiMeldungen.size() - 1;
    _lMultiMeldungen.remove(letzteElementPosition);

  }

  public void setStatus(IStatus status)
  {
    // TODO Auto-generated method stub
    _aktuelleMeldung = (MultiStatus)status;
    _aktuellerZeitstempel = System.currentTimeMillis();
  }

  @Override
  public void konsoleAusgabeUpdate(KonsoleAusgabeUpdateEvent e)
  {
    IStatus status = e.getStatus();
   

    if (_aktuelleMeldung != null)
    {

      if (canFinfish)
      {
        hinzufuegenMultimeldung(_aktuelleMeldung, _aktuellerZeitstempel);
        canFinfish = false;
      }
      else
      {
        if (_aktuelleMeldung.isMultiStatus())
        {
          MultiStatus multiStatus = (MultiStatus)_aktuelleMeldung;

          multiStatus.merge(new Status(status.getSeverity(), status.getPlugin(), status.getMessage()));
          
          _aktuelleMeldung = multiStatus;
          
//          canFinfish = true;
        }
        else
        {
//          System.out.println("Kein MultiStatus: " + _aktuelleMeldung);
        }
      }
    }
  }
  
  public String getKonfiguration()
  {   
    return _konfiguration;
  }
  
  public void setKonfiguration(String konfiguration)
  { 
    _konfiguration = konfiguration;
  }

}
