/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.datenbereich.editor;

import de.bsvrz.dav.daf.main.Data;

/**
 * Kapselt alle notwendigen Daten zur eindeutigen Identifikation eines
 * Datensatzes zu einer Attributgruppe oder einer Menge.
 * 
 * @author Krause
 * 
 */
public class DatenBereichEditorKapsel implements Comparable<String>
{ 
  /**
   * Elternteil {@link DatenBereichSOKapsel}, das bergeordnet ist
   */
  private DatenBereichSOKapsel _elternKapsel = null;

  /**
	 * Name der Kapsel, entspricht
	 * Pid der Attributgruppe bzw. Name der Menge
	 */
	private String _name = null;
	
	/**
	 * Typ der Kapsel
	 */
	public static enum TYP {Atg, Menge};

	private TYP _typ = null;
	
	/**
	 * Pid des Aspektes fr den Empfang
	 */
	private String _pidAspEmpfang = null;

	/**
	 * Pid des Aspekts zum Senden
	 */
	private String _pidAspSenden = null;

	/**
	 * Simulationsvariante
	 */
	private short _simVar = 0;

	/**
	 * Data Empfangen
	 */
	private Data _dataEmpfangen = null;

	/**
	 * Data Gesendet
	 */
	private Data _dataSenden = null;

  /**
   * Konstruktor. bernimmt die notwendigen Parameter zur eindeutigen
   * Identifikation eines Datensatzes zu einer Attributgruppe oder einer Menge.
   * 
   * @param elternKapsel
   *          Elternteil dieser Kapsel
   * @param name
   *          Name der Kapsel, entspricht je nach Typ:
   *          Pid der Attributgruppe bzw. Name der Menge
   * @param pidAspEmpfang
   *          Pid des Aspektes zum Empfang
   * @param pidAspSenden
   *          Pid des Aspektes zum Senden
   * @param simVar
   *          Simulationsvariante
   * @param typ
   *          Typ der Kapsel
   */
  public DatenBereichEditorKapsel(DatenBereichSOKapsel elternKapsel, String name,
      String pidAspEmpfang, String pidAspSenden, short simVar, TYP typ)
  {
    this._elternKapsel = elternKapsel;
    this._name = name;
    this._pidAspEmpfang = pidAspEmpfang;
    this._pidAspSenden = pidAspSenden;
    this._simVar = simVar;
    this._typ = typ;
   
//    System.out.println("ElternKapsel: " + elternKapsel.getPidSystemObjekt());
    
    if ( elternKapsel != null )
    {
      ((DatenBereichSOKapsel) elternKapsel).addKind( this );
    }
  }
  
  /**
   * Konstruktor. bernimmt die notwendigen Parameter zur eindeutigen
   * Identifikation eines Datensatzes zu einer Attributgruppe.
   * 
   * @param elternKapsel
   *          Elternteil dieser Kapsel
   * @param pidAtg
   *          Pid der Attributgruppe
   * @param pidAspEmpfang
   *          Pid des Aspektes zum Empfang
   * @param pidAspSenden
   *          Pid des Aspektes zum Senden
   * @param simVar
   *          Simulationsvariante
   */
  public DatenBereichEditorKapsel(DatenBereichSOKapsel elternKapsel, String pidAtg,
      String pidAspEmpfang, String pidAspSenden, short simVar)
  {
    this(elternKapsel, pidAtg, pidAspEmpfang, pidAspSenden, simVar, TYP.Atg);
  }
  
  /**
   * Konstruktor. bernimmt die notwendigen Parameter zur eindeutigen
   * Identifikation eines Datensatzes zu einer Attributgruppe oder einer Menge. <br>
   * Der Aspekt wird fr die Empfangs- und Sendeanmeldung verwendet. <br>
   * Als Simulationsvariante wird 0 verwendet.
   * 
   * @param elternKapsel
   *          Elternteil dieser Kapsel
   * @param name
   *          Name der Kapsel (Pid der Attributgruppe oder Name der Menge)
   * @param pidAsp
   *          Pid des Aspektes zum Empfang und Senden
   * @param typ
   *          Typ der Kapsel          
   */
  public DatenBereichEditorKapsel(DatenBereichSOKapsel elternKapsel, String name, String pidAsp, TYP typ)
  {
    this(elternKapsel, name, pidAsp, pidAsp, (short)0, typ);
  }

  /**
   * Konstruktor. bernimmt die notwendigen Parameter zur eindeutigen
   * Identifikation eines Datensatzes zu einer Attributgruppe. <br>
   * Der Aspekt wird fr die Empfangs- und Sendeanmeldung verwendet. <br>
   * Als Simulationsvariante wird 0 verwendet.
   * 
   * @param elternKapsel
   *          Elternteil dieser Kapsel
   * @param pidAtg
   *          Pid der Attributgruppe bzw. Name der Menge
   * @param pidAsp
   *          Pid des Aspektes zum Empfang und Senden
   */
  public DatenBereichEditorKapsel(DatenBereichSOKapsel elternKapsel, String pidAtg, String pidAsp)
  {
    this(elternKapsel, pidAtg, pidAsp, pidAsp, (short)0, TYP.Atg);
  }
  
	/**
	 * Liefert die Pid des SystemObjektes des Elternteils.
	 * 
	 * @return Pid des SystemObjektes
	 */
	public String getPidSystemObjekt()
	{
	  if (_elternKapsel != null)
	    return _elternKapsel.getPidSystemObjekt();
	  else
	    return "";
	  
	}
 
	/**
	 * Liefert den Namen der Kapsel
	 * Dieser entspricht der Pid der Attributgruppe
	 * oder dem Name der Menge.
	 * 
	 * @return Pid der Attributgruppe
	 */
	public String getName()
	{
		return _name;
	}

	/**
	 * Liefert die Pid des Aspektes zum Empfangen.
	 * 
	 * @return Pid des Aspektes zum Empfangen
	 */
	public String getPidAspEmpfang()
	{
		return _pidAspEmpfang;
	}

	/**
	 * Liefert die Pid des Aspektes zum Senden.
	 * 
	 * @return Pid des Aspektes zum Senden
	 */
	public String getPidAspSenden()
	{
		return _pidAspSenden;
	}

	/**
	 * Liefert die Simulationsvariante.
	 * 
	 * @return Simulationsvariante
	 */
	public short getSimulationsVariante()
	{
		return _simVar;
	}

	/**
	 * Liefert das {@link Data}-Objekt.
	 * 
	 * @return {@link Data}-Objekt
	 */
	public Data getDataEmpfangen()
	{
		return _dataEmpfangen;
	}

	/**
	 * Setzt das {@link Data}-Objekt.
	 * 
	 * @param data
	 *          {@link Data}-Objekt
	 */
	public void setDataEmpfangen(Data data)
	{
		this._dataEmpfangen = data;
	}

	/**
	 * Liefert die Daten, die gesendet werden sollen.
	 * 
	 * @return Zu sendende Daten.
	 */
	public Data getDataSenden()
	{
		return this._dataSenden;
	}

	/**
	 * Setzt die Daten, die gesendet werden sollen.
	 * 
	 * @param dataSenden
	 *          Zu sendende Daten.
	 */
	public void setDataSenden(Data dataSenden)
	{
		this._dataSenden = dataSenden;
	}

	public String toString()
  {
    String re = "";
    String pidSystemObjekt = null;
    if ( _elternKapsel != null )
      pidSystemObjekt = _elternKapsel.getSystemObjekt().getPid();

    if ( _typ == TYP.Atg )
      re = re + "Pid SystemObjekt: " + pidSystemObjekt + "\nPid ATG: " + _name + "\nPid ASP: "
          + _pidAspEmpfang;
    else
      re = re + "Pid SystemObjekt: " + pidSystemObjekt + "\nName Menge: " + _name + "\nPid ASP: "
          + _pidAspEmpfang;

    return re;
  }

	/**
	 * Liefert die ID dieser Kapsel bestehend aus
	 * <code>Pid Systemobjekt;Pid_ATG;Pid_ASP_Empfang;Pid_ASP_Senden;Simulationsvariante</code>
	 * 
	 * @return ID dieser Kapsel
	 */
	public String getId()
	{
		String id = "";

		if (_elternKapsel != null)
		  id = id + _elternKapsel.getSystemObjekt().getPid() + ",";
		else
		  id = id + " ,";
		
		id = id + _name + ",";
		id = id + _pidAspEmpfang + ",";
		id = id + _pidAspSenden + ",";
		id = id + _simVar;

		return id;
	}

	/**
	 * Baut aus der ID einer {@link DatenBereichEditorKapsel} eine neue
	 * {@link DatenBereichEditorKapsel} auf.
	 * 
	 * @param id
	 *          ID der {@link DatenBereichEditorKapsel}.
	 * @return Aus ID generierte {@link DatenBereichEditorKapsel}.
	 */
	public static DatenBereichEditorKapsel baueKapselAusId(String id)
	{
		DatenBereichEditorKapsel re = null;

		String[] split = id.split(",");

		if (split.length == 5)
		{
//			String pidSystemObject = split[0];
			String _nameKapsel = split[1];
			String pidAspEmpfang = split[2];
			String pidAspSenden = split[3];
			String strSimVar = split[4];
			short simVar = 0;
			try
			{
				simVar = Short.valueOf(strSimVar);
			}
			catch (NumberFormatException nfe)
			{
				nfe.printStackTrace();
			}

			re = new DatenBereichEditorKapsel(null, _nameKapsel, pidAspEmpfang,
					pidAspSenden, simVar);
		}

		return re;
	}

  /**
   * Liefert den Elternteil der Kapsel.
   * 
   * @return Elternteil der Kapsel
   */
  public DatenBereichSOKapsel getElternKapsel()
  {
    return _elternKapsel;
  }
	
	@Override
	public int compareTo(String o)
	{
		int re = 0;

		if (_elternKapsel != null && _elternKapsel.getPidSystemObjekt()!= null)
		{
			re = _elternKapsel.getPidSystemObjekt().compareTo(o);
		}

		return re;
	}

	public boolean isTypAtg()
	{
	  if (_typ == TYP.Atg)
	    return true;
	  else
	    return false;
	}

	 public boolean isTypMenge()
	  {
	    if (_typ == TYP.Menge)
	      return true;
	    else
	      return false;
	  }

}
