/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

/**
 * Hierarchie aus Hierarchieobjekten
 * 
 * @author Sans
 *
 */
public class HOHierarchie
{
  /**
   * Wurzelobjekt der Hierachie
   */
  HierarchieObjekt _wurzel = null;
    
  /**
   * Bestimmt das Wurzelobjekt der Hierarchie
   * 
   * @return as Wurzelobjekt der Hierarchie
   */
  public HierarchieObjekt getWurzel()
  {
    return _wurzel;
  }
  
  /**
   * Map, enthlt fr jedes in der Hierarchie enthaltene Objekt die Liste der untergeordenten Objekte
   */
  private Map<HierarchieObjekt, List<HierarchieObjekt>> _untergeordneteObjekte = 
    new HashMap<HierarchieObjekt, List<HierarchieObjekt>>();
    
  /**
   * Map, enthlt fr jedes in der Hierarchie enthaltene Hierarchieobjekt das bergeordnete Objekt 
   */
  private  Map<HierarchieObjekt, HierarchieObjekt> _uebergeordneteObjekte = 
    new HashMap<HierarchieObjekt, HierarchieObjekt>();
    
  /**
   * Bestimmt die untergeordneten Hierarchieobjekte zu einem Objekt der Hierarchie
   * 
   * @param ho das Objekt, zu dem die untergordenten Objekte bestimmt werden sollen
   * @return Liste der untergeordneten Objekte zum bergebenen Objekt
   */
  public List<HierarchieObjekt> getUntergeordneteObjekte(HierarchieObjekt ho)
  {
    List<HierarchieObjekt> result = _untergeordneteObjekte.get(ho);
    
    if (result == null)
    {
      result = Collections.emptyList();
    }
    
    return result;
  }
  
  /**
   * Bestimmt das bergeordnete Objekt zu einem Hierachie
   * 
   * @param ho das Hierarchieobjekt, zu dem das bergeordnete Objekt bestimmt werden soll
   * @return das bergeordnete Objekt zum bergebenen Hierarchieobjekt
   */
  public HierarchieObjekt getUebergeordnetesObjekt(HierarchieObjekt ho)
  {
    return _uebergeordneteObjekte.get(ho);
  }
    
  /**
   * Erzeugt eine neue Instanz vom Typ HOHierarchie aus Hierarchieobjekten einer Instanz
   * vom Typ IHierarchie
   * Es werden das bergebenen Wurzelobjekt und alle untergeordenten Objekte in die neue
   * Instanz bernommen
   * 
   * @param wurzel das Wuurzelobjekt
   * @param hierarchie die Hierachie, die das Wurzelobjekt und die zu bernehmenden 
   * Hierachieobjekte enthlt
   */
  public HOHierarchie(HierarchieObjekt wurzel, IHierarchie hierarchie)
  {
    _wurzel = wurzel;
    hinzufuegen(wurzel, hierarchie);
  }

  /**
   * Fgt ein Hierachieobjekt und rekursiv alle seine untergeordneten Objekte aus einer
   * Instanz vom Typ IHierarchie dieser Hierarchie hinzu
   * 
   * @param ho das hinzuzufgende Objekt
   * @param hierarchie die Hierarchie, die das hinzuzufgende Objekt enthlt
   */
  private void hinzufuegen(HierarchieObjekt ho, IHierarchie hierarchie)
  {
    for (HierarchieObjekt kind: hierarchie.getUntergeordneteObjekte( ho ))
    {
      objektHinzufuegen( ho, kind );
      hinzufuegen(kind, hierarchie);
    }
  }
    
  /**
   * Entfernt ein Hierarchieobjekt rekursiv mit allen untergeordneten Objekten aus 
   * dieser Hierarchie
   * 
   * @param ho das zu entferndende Objekt
   */
  public void objektEntfernen(HierarchieObjekt ho)
  {
    List<HierarchieObjekt> liste = _untergeordneteObjekte.get(ho);
    if (liste != null)
    {
      for (HierarchieObjekt kind: liste)
      {
        objektEntfernen(kind);
      }
    }
    
    HierarchieObjekt vater = _uebergeordneteObjekte.get(ho);
    if (vater != null)
    {  
      objektEntfernen(vater, ho);
    }  
  }
  
  /**
   * Intern benutzte Methode zum Einfgen eine Hierarchieobjekts in die Hierarchie
   * unterhalb eines bergeordneten Objekts
   * 
   * @param vater das bergeordnete Objekts, unter dem das Hierarchieobjekt eingefgt werden soll
   * @param kind das einzufgende Hierarchieobjekt
   */
  private void objektHinzufuegen(HierarchieObjekt vater, HierarchieObjekt kind)
  {
    List<HierarchieObjekt>liste = _untergeordneteObjekte.get(vater);
    
    if (liste == null)
    {
      liste = new Vector<HierarchieObjekt>();
      _untergeordneteObjekte.put(vater, liste);
    }
    
    liste.add(kind);
    
    _uebergeordneteObjekte.put(kind, vater);
  }
  
  /**
   * Intern benutzte Methode zum Entfernen eine Hierarchieobjekts in die Hierarchie
   * unterhalb eines bergeordneten Objekts
   * 
   * @param vater das bergeordnete Objekts, unter dem das Hierarchieobjekt entfernt werden soll
   * @param kind das zu entferndende Hierarchieobjekt
   */
  private void objektEntfernen(Object vater, HierarchieObjekt kind)
  {
    List<HierarchieObjekt>liste = _untergeordneteObjekte.get(vater);
    
    if (liste != null)
    {
      liste.remove(kind);
    }
    
    Object objekt = _uebergeordneteObjekte.get(kind);
    if (objekt == vater)
    {
      _uebergeordneteObjekte.remove(kind);
    }
  }
 
}