/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Das Hierachieobjekt (HO) fr die Teilkonfigurationsassistenten wird modelliert
 * 
 * @author Sans
 *
 */
public class HierarchieObjekt
{
  /**
   * Der Hierachieobjekt-Typ dieses HO
   */
  private IHierarchieObjektTyp _hierachieObjektTyp ;
  
  /**
   * Setzt den Hierachieobjekt-Typ dieses HO
   * 
   * @param hierachieObjektTyp der Hierachieobjekt-Typ dieses HO
   */
  public void setHierarchieObjektTyp(IHierarchieObjektTyp hierachieObjektTyp)
  {
    _hierachieObjektTyp = hierachieObjektTyp; 
  }
  
  /**
   * Bestimmt den Hierachieobjekt-Typ dieses HO
   * 
   * @return den Hierachieobjekt-Typ dieses HO
   */
  public IHierarchieObjektTyp getHierarchieObjektTyp()
  {
    return _hierachieObjektTyp;
  }
  
  /**
   * Die Systemobjekte, die dieses HO bilden
   */
  private List<SystemObject> _systemObjekte = Collections.emptyList();
  
  /**
   * Setzt die Systemobjekte, die dieses HO bilden 
   * 
   * @param objekte die Systemobjekte, die dieses HO bilden 
   */
  public void setSystemObjekte(SystemObject[] objekte)
  {
    if (objekte != null && objekte.length > 0)
    {
      _systemObjekte = new LinkedList<SystemObject>(Arrays.asList(objekte));
    }
    else
    {
      _systemObjekte = Collections.emptyList();
    }
  }
  
  /**
   * Bestimmt die Systemobjekte, die dieses HO bilden
   * 
   * @return Die Systemobjekte, die dieses HO bilden
   */
  public List<SystemObject> getSystemObjekte()
  {
    return Collections.unmodifiableList(_systemObjekte);
  }

  /**
   * Setzt ein neues Systemobjekt
   * 
   * @param index Index, an den das Systemobjekt gesetzt werden soll
   * @param so das neue Systemobjekt
   */
  public void setSystemObjekt(int index, SystemObject so)
  {
    _systemObjekte.set(index, so);
  }
  
  /**
   * Bestimmt die Systemobjekte an einer Position
   * 
   * @param index Index, fr den da Systemobjekt bestimmt werden soll
   * @return Das Systemobjekte an der Position index
   */
  public SystemObject getSystemObjekt(int index)
  {
    return _systemObjekte.get(index);
  }
  
  /**
   * Erzeugen eines neuen HO
   * 
   * @param  hierachieObjektTyp der Hierachieobjekt-Typ dieses HO
   * @param objekte die Systemobjekte, die dieses HO bilden
   */
  public HierarchieObjekt(IHierarchieObjektTyp hierachieObjektTyp, SystemObject[] objekte,
      IHierarchieObjektTyp[] typSortierFolge)
  {
    setHierarchieObjektTyp(hierachieObjektTyp);
    setSystemObjekte(objekte);
    setTypSortierFolge(typSortierFolge);
  }
  
  /**
   * Liste, die angibt, in welcher Reihenfolge die relevanten Hierarchieobjekttypen fr dieses
   * Hierarchieobjekt sortiert werden sollen
   */
  private IHierarchieObjektTyp[] _typSortierFolge = null; 
  
  /**
   * Bestimmt die HOT-Sortierreihenfolge fr dieses Hierarchieobjekt
   * 
   * @return die HOT-Sortierreihenfolge fr dieses Hierarchieobjekt
   */
  public List<IHierarchieObjektTyp> getTypSortierFolge()
  {
    if (_typSortierFolge != null)
    {  
      return Collections.unmodifiableList(Arrays.asList(_typSortierFolge));
    }
    else
    {
      return Collections.emptyList();
    }
      
  }

  /**
   * Setzt die HOT-Sortierreihenfolge fr dieses Hierarchieobjekt
   * 
   * @param typSortierFolge die zu setzende HOT-Sortierreihenfolge fr dieses Hierarchieobjekt
   */
  public void setTypSortierFolge(IHierarchieObjektTyp[] typSortierFolge)
  {
    _typSortierFolge = typSortierFolge;
  }

  /**
   * Die Text-Reprsentation eines Hierarchieobjekts ist die PID seines ersten (Index 0) 
   * Systemobjekts
   * 
   * @return die PID des ersten (Index 0) Systemobjekts dieses Hierarchieobjekts
   */
  @Override
  public String toString()
  {
    //return getHierarchieObjektTyp() + (getSystemObjekte().size() > 0 ? " " + getSystemObjekte().get(0).getPid() : "");
    return getSystemObjekte().size() > 0 ? getSystemObjekte().get(0).getPid() : "";
  }
  
  /**
   * Bestimmt, ob fr ein Systemobjekt die Rechte bestehen, um es zu ndern
   *  
   * @param so das Systemobjekt, fr das die Rechte bestimmt werden sollen
   * @return <code>true</code>, wenn das Systemobjekt gendert werden kann, <code>false</code> sonst
   */
  public static boolean isSystemObjektAenderbar(SystemObject so)
  {
    return (so.getDataModel().getConfigurationAuthority() == 
      so.getConfigurationArea().getConfigurationAuthority());
  }
  
  /**
   * Bestimmt, ob fr ein Systemobjekt die Daten einer Attributgruppe aufgrund der Attributgruppenverwendung 
   * nderbar sind, aussedem sind Attributgruppen-Daten fr noch nicht aktivierte Systemobjekte immer nderbar
   *  
   * @param so das Systemobjekt, fr das die Rechte bestimmt werden sollen
   * @return <code>true</code>, wenn das Systemobjekt gendert werden kann, <code>false</code> sonst
   * 
   * @param so das Systemobjekt, fr dessen Attributgruppe die nderbarkeit der Daten bestimmt werden soll
   * @param atg die Attributgruppe fr die nderbarkeit der Daten bestimmt werden soll
   * @return <code>true</code>, wenn das Systemobjekt nicht aktiviert ist oder die Attributgruppenverwendung
   * Datennderung zulsst, <code>false</code> sonst
   */
  public static boolean isAttributgruppeAenderbar(SystemObject so, AttributeGroup atg)
  {
    AttributeGroupUsage atgUsage = atg.getAttributeGroupUsage(so.getDataModel().getAspect("asp.eigenschaften"));
    if (atgUsage != null)
    {
      ConfigurationObject ko = (ConfigurationObject)so;
      if (ko.getValidSince() < ko.getConfigurationArea().getModifiableVersion()) // Objekt ist bereits aktiviert
      {
        AttributeGroupUsage.Usage usage = atgUsage.getUsage();
        return (usage != AttributeGroupUsage.Usage.OptionalConfigurationData &&
            usage != AttributeGroupUsage.Usage.RequiredConfigurationData);
      }
    }
    return true;
  }
  
}
