/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Collection;

import org.eclipse.core.runtime.IStatus;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Die Schnittstelle IHierarchieManager modelliert die Manipulation von Hierarchieobjekten
 * innnerhalb einer Hierarchie vom Typ IHierarchie
 * 
 * @author Sans
 *
 */
public interface IHierarchieManager
{
  /**
   * Bestimmt die zugrundeliegende Hierarchie
   * 
   * @return die zugrundeliegende Hierarchie
   */
  public IHierarchie getHierarchie();
  
  /**
   * Setzt die zugrundeliegende Hierarchie
   * 
   * @param hierachie die zu setzende Hierarchie
   */
  public void setHierarchie(IHierarchie hierachie);
  
  /**
   * Bestimmt - aus dem der Hierarchie zugrundeliegendem Datenmodell - rekursiv alle
   * dem bergebenen Objekt untergeordneten Objekte, erzeugt entsprechende Hierarchieobjekte
   * und fgt diese in die Hierarchie ein
   * 
   * @param objekt das Objekt, zu dem die unterordneten Objekte bestimmt und Hierarchieobjekte 
   * erzeugt werden, die dann in die Hierarchie eingefgt werden
   */
  public void bestimmeUntergeordneteHOs(Object objekt);
  
  /**
   * Bestimmt die Liste der Hierarchieobjekttypen, die in der Hierarchieebene unter der Ebene des 
   * bergebenen Objekts vewendet werden knnen
   * 
   * @param objekt das Objekt in der Hierarchie, von dessen untergeordneter Ebene die HOT bestimmt
   * werden sollen
   * @return die Liste der HOT, die in der Ebene unter objekt verwendet werden knnen
   */
  public IHierarchieObjektTyp[] getUntergeordneteHOTs(Object objekt);
  
  /**
   * Erzeugen eines neuen Hierachieobjekts in der Hierarchie
   * 
   * @param kb der Konfigurationsbereich, in dem das neue Objekt erzeugt wird
   * @param vater Eltern-Objekt des zu erzeugenden HO
   * @param pid Liste der PID fr die zu erzeugenden Systemobjekte des neuen HO
   * @param hot Hierarchieobjekttyp, den das neue HO erhalten soll
   * @paran pruefePid bestimmt, ob die bergebenen Pid explizit geprft werden
   * @return Eine Instanz vom Typ HOMitStatus, die den Status der Operation enthlt, sowie bei Erfolg
   * das neue HO.
   */
  public HOMitStatus erzeugeNeuesHO(ConfigurationArea kb, Object vater, String pid[], IHierarchieObjektTyp hot,
      boolean pruefePid);
  
  /**
   * Kopiert ein Hierarchieobjekt von einem Eltern-Objekt unter ein anderes. Die Kopie erfolgt
   * gemss Anforderungen TKA-23
   * 
   * @param ho das zu kopierende HO
   * @param kb der Konfigurationsbereich, in dem das neue Objekt erzeugt wird
   * @param neuerVater das neue Elternobjket fr das kopierte HO
   * @param neuePid die Liste der PID fr die kopierten Systemobjekte des HO
   * @param rekursiv Flag, das angibt ob untergeordnete Objekte rekursiv mitkopiert werden
   * @param isWurzel Flag, das angibt, ob bei Rekursion der aktuelle Aufruf der erste Aufruf ist
   * @return Eine Instanz vom Typ HOMitStatus, die den Status der Operation enthlt, sowie bei Erfolg
   * das kopierte HO.
   */
  public HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, 
      String neuePid[], boolean rekursiv, boolean isWurzel);
  
  /**
   * Kopiert ein Hierarchieobjekt von einem Eltern-Objekt unter ein anderes. Die Systemobjekte
   * des HO werden dabei jeweils in identische Systemobjekte mit identischer PID kopiert.
   * Das bergebene HO und seine Systemobjekte werden danach gelscht.
   * 
   * @param ho das zu verschiebende HO
   * @param neuerVater das neue Elternobjket fr das verschobene HO
   * @param rekursiv Flag, das angibt, ob das Kopieren rekurisv erfolgen soll
   * @param isWurzel Flage, das angibt, ob dieser Aufruf der initiale Aufruf einer Rekursion 
   * (intern oder extern) ist
   * @return Eine Instanz vom Typ HOMitStatus, die den Status der Operation enthlt, sowie bei Erfolg
   * das verschobene HO.
   */
  public HOMitStatus verschiebeHO(HierarchieObjekt ho, Object neuerVater);
  
  /**
   * Ein Hierarchieobjekt wird gelscht, sowie rekursiv alle untergeordneten Hierarchieobjekte in der
   * Hierarchie. Fr jedes Hierarchieobjekt werden die Systemobjekte gelscht, das Hierarchieobjekt wird
   * aus der Hierarchie entfernt
   * 
   * @param ho das zu lschende Hierarchieobjekt
   * @return den Status der Operation
   */
  public IStatus loescheHO(HierarchieObjekt ho);
  
  /**
   * Die Daten, die fr ein Hierarchieobjekts bernommen werden sollen, werden gem einer zu implementierenden 
   * Instanz vom Typ IPruefungen geprft
   * 
   * @param ho das Hierarchieobjekt, fr das Daten geprft werden sollen
   * @param daten die zu prfenden Daten
   * @return den Status der Prfungen
   */
  public IStatus pruefeDaten(HierarchieObjekt ho, HODaten daten);
  
  /**
   * Daten fr ein Hierarchieobjekts werden bernommen
   * 
   * @param ho das Hierarchieobjekt, fr das Daten bernommen werden sollen
   * @param daten die zu bernehmenden Daten
   * @return den Status der Operation
   */
  public IStatus uebernehmeDaten(HierarchieObjekt ho, HODaten daten);
  
  /**
   * Prft ob die Neuanlage zu einem Hierarchieobjekttyp im Objekt erlaubt ist
   * 
   * @param objekt  das Objekt in der Hierarchie, in dem die Neuanlage erfolgen soll
   * @param hot     der HOT zu dem die Neuanlage erfolgen soll
   * @return true, wenn die Neuanlage erlaubt ist
   */
  public boolean pruefeNeuanlageVerboten(Object objekt, IHierarchieObjektTyp hot);
  
  
  /**
   * Ermittelt die Liste von HOTs bei denen das Entstehen von offenen Referenzen 
   * durch das Lschen eines konkreten HO vom bergebenen HOT  verboten werden soll.   
   * 
   * @param der HOT des konkret zu lschenden HO
   * @return die Liste der HOT
   */
  public Collection<SystemObjectType> loeschenOffeneReferenzenVerboten(IHierarchieObjektTyp hot);
  
  
  /**
   * Zeigt an ob die Prfung auf offene Referenzen, die zu loeschenden Objekte bercksichtigen soll
   * 
   * @return true, wenn die Prfung auf offene Referenzen, die zu loeschenden Objekte bercksichtigen soll
   */
  public boolean verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen();

  /**
   * Zeigt an, ob das bergebene Hierarchieobjekt beim rekursiven Kopieren als untergeordnetes 
   * Objekt mitkopiert werden soll oder nicht
   *  
   * @param ho das aktuell zu kopierende HO
   * @return <code>true</code>, wenn das aktuell zu kopierende Objekt nicht kopiert werden soll, 
   * <code>false</code> (default) sonst
   */
  public boolean kopierenNichtErlaubt(HierarchieObjekt ho);
  
}
