/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;


/**
 * Die Daten (Systemobjekdaten, ATG-Datenstze, Mengendaten) der Hierarchieobjekte 
 * einer Teilhierarchie vom Typ <code>HOHierarchie<7code> knnen zwischengespeichert und dann in
 * der ursprnglichen Hierarchie gelscht werden.<br>
 * Die zwischengespeicherten Daten knnen in neue Hierarcharchieobjekte der ursprnglichen Hierarchie
 * kopiert werden
 * 
 * @author Sans
 */
public class ZwischenSpeicher
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Zu bearbeitende Teilhierarchie, die eine Kopie einer ursprnglichen Hierarchie darstellt
   */
  protected final HOHierarchie _hierarchie;

  /**
   * Hierarchiemanager fr die ursprngliche Hierarchie
   */
  protected final IHierarchieManager _hm;
  
  /**
   * Speichert die zwischengespeicherten Daten
   */
  protected final Map<HierarchieObjekt, HODaten> _hoDaten = new HashMap<HierarchieObjekt, HODaten>();
  
  
  /**
   * Erzeugt eine neue Instanz vom Typ <code>ZwischenSpeicher<7code>
   * 
   * @param hierarchie zu bearbeitende Teilhierarchie, die eine Kopie einer ursprnglichen Hierarchie 
   * darstellt
   * @param hm Hierarchiemanager fr die ursprngliche Hierarchie
   */
  public ZwischenSpeicher(HOHierarchie hierarchie, IHierarchieManager hm)
  {
    _hierarchie = hierarchie;
    _hm = hm;
  }
  
  public IStatus speichernUndLoeschenRekursiv(HierarchieObjekt ho)
  {
    MultiStatus status = new MultiStatus(Activator
        .PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + ho +
        " vom Typ " + ho.getHierarchieObjektTyp(), null);
    
    IHierarchie hierarchie = _hm.getHierarchie(); 
    
    while (!hierarchie.getUntergeordneteObjekte(ho).isEmpty())
    {
      HierarchieObjekt kind = hierarchie.getUntergeordneteObjekte(ho).get(0);
      
      status.add(speichernUndLoeschenRekursiv(kind));
      
      if (status.matches(IStatus.CANCEL | IStatus.ERROR))
      {
        break;
      }
    }
    
    erzeugeZwischengespeicherteDaten(ho);
    
    if (!status.matches(IStatus.CANCEL | IStatus.ERROR))
    {  
      status.merge(_hm.loescheHO(ho));
    }
    
    return status;
  }
  
  /**
   * Erzeugen und speichern der zwischengespeicherten Daten
   * 
   * @param ho das HO in der Teilhierarchie, fr das Daten zwischengespeichert werden 
   */
  protected void erzeugeZwischengespeicherteDaten(HierarchieObjekt ho)
  {
  // Erzeuge temporre Daten
    HODaten hoDaten = new HODaten();
    for (SystemObject so: ho.getSystemObjekte())
    {
      SystemObjektDaten soDaten = new SystemObjektDaten();
      soDaten.setPid(so.getPid());
      soDaten.setName(so.getName());
            
      ConfigDataModel dataModel = _hm.getHierarchie().getDataModel();
      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
      
      // ATG-Daten
      for (AttributeGroupUsage atgu: so.getUsedAttributeGroupUsages())
      {
        AttributeGroup atg = atgu.getAttributeGroup();
        Data data = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);
        if (data != null)
        {
          soDaten.setAtgDaten(atg.getPid(), data);
        }
      }
      
      // Mengen
      for (ObjectSet menge: ((ConfigurationObject)so).getObjectSets())
      {
        if (menge instanceof NonMutableSet)
        {
          soDaten.setMengenElemente(menge.getName(), ((NonMutableSet)menge).getElementsInModifiableVersion());
        }
        else if (menge instanceof MutableSet)
        {
          soDaten.setMengenElemente(menge.getName(), menge.getElements());
          
          AttributeGroup atg = dataModel.getAttributeGroup("atg.dynamischeMenge");
          if (atg != null)
          {  
            Data data = ((ConfigSystemObject)menge).getConfigurationData(atg, lookup);
            if (data != null)
            {
              soDaten.setDynMengenDaten(menge.getName(), data);
            }  
          }  
        }
      }
      
      hoDaten.setSystemObjektDaten(so, soDaten);
    }
    
    _hoDaten.put(ho, hoDaten);
  }
  
  /**
   * Kopiert die zwischengespeicherten Daten (Systemobjektdaten, ATG-Daten, Mengendaten) in ein neues HO
   * 
   * @param ho das neue HO
   * @param zwischengespeichertesHO das HO, fr das in der Teilhierarchie Daten zwischengespeichert wurden
   * @return den Status der Aktion
   */
  protected IStatus kopiereZwischengespeichertDaten(HierarchieObjekt ho, HierarchieObjekt zwischengespeichertesHO)
  {
    HODaten hoDaten = _hoDaten.get(zwischengespeichertesHO);
    
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten fr Hierarchieobjekt " +
        zwischengespeichertesHO, null);
    
    if (hoDaten != null)
    {
      int index = 0;
      
      for (SystemObject lauf: hoDaten.getSystemObjekte())
      {
        SystemObjektDaten soDaten = hoDaten.getSystemObjektDaten(lauf);
        Assert.isTrue(soDaten != null);
        SystemObject so = ho.getSystemObjekt(index);
        Assert.isTrue(so.getPid().equals(soDaten.getPid()));
        DataModel dataModel = _hm.getHierarchie().getDataModel();
        MultiStatus unterStatus = status;
        
        if (hoDaten.getSystemObjekte().size() > 0)
        {
          unterStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten fr Systemobjekt " +
              soDaten.getPid(), null);
        }
        
        try
        {
          so.setName(soDaten.getName());
          
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Setze Name \"" + soDaten.getName() + "\" fr Systemobjekt "
              + so.getPid()));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Name \"" + soDaten.getName() + "\" fr Systemobjekt "
              + so.getPid() + " konnte nicht gesetzt werden", e));
        }
        
        for (String atgName: soDaten.getAtg())
        {
          AttributeGroup atg = dataModel.getAttributeGroup(atgName);
          if (atg != null)
          {  
            Data data = soDaten.getAtgDaten(atgName);

            try
            {
              so.setConfigurationData(atg, data);
              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Kopiere Daten der ATG "
                  + atg.getPid() + " fr Systemobjekt " + so.getPid()));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Daten der ATG "
                  + atg.getPid() + " fr Systemobjekt " + so.getPid() + " konnten nicht kopiert werden", e));
            }
          }
          else
          {
            _debug.warning("Attributgruppe " + atgName + " ist nicht vorhanden");
          }
        }
        
        for (String mengenName: soDaten.getDynMengen())
        {
          ObjectSet menge = ((ConfigurationObject)so).getObjectSet(mengenName);
          if (menge != null)
          {
            AttributeGroup atg = dataModel.getAttributeGroup("atg.dynamischeMenge");
            if (atg != null)
            {  
              Data data = soDaten.getDynMengenDaten(mengenName);
              try
              {
                menge.setConfigurationData(atg, data);
                status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Kopiere Daten der ATG "
                    + atg.getPid() + " fr die Menge "  +  mengenName + " am Systemobjekt " + so.getPid()));
              }
              catch (ConfigurationChangeException e)
              {
                status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Daten der ATG "
                    + atg.getPid() + " fr die Menge "  +  mengenName + " am Systemobjekt " + so.getPid(), e));
              }
            }
            else
            {
              _debug.warning("Attributgruppe atg.dynamischeMenge ist nicht vorhanden");
            }
          }
          else
          {
            status.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "MengeName \"" + mengenName + 
                "\" ist am Systemobjekt " + so.getPid() + " nicht vorhanden"));
          }
        }
        
        for (String mengenName: soDaten.getMengen())
        {
          List<SystemObject> elementListe = soDaten.getMengenElemente(mengenName);
          if (elementListe != null && elementListe.size() > 0)
          {
            ObjectSet menge = ((ConfigurationObject)so).getObjectSet(mengenName);
            if (menge != null)
            {
              for (SystemObject element: elementListe)
              {
                try
                {
                  menge.add(element);
                  status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + element.getPid() +
                      " der  Menge "  +  mengenName + " am Systemobjekt " + so.getPid() + " hinzugefgt"));
                }
                catch (ConfigurationChangeException e)
                {
                  status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + element.getPid() +
                      " konnte der  Menge "  +  mengenName + " am Systemobjekt " + so.getPid() + 
                      " nicht hinzugefgt werden ", e));
                }
              }
            }
            else
            {
              status.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "MengeName \"" + mengenName + 
                  "\" ist am Systemobjekt " + so.getPid() + " nicht vorhanden"));
            }
          }
        }
        
        index += 1;
        if (unterStatus != status)
        {
          status.add(unterStatus);
        }
      }
    }
    else
    {
      _debug.warning("Fr Hierarchieobjekt " + ho + " wurden keine zwischengespeicherte Daten gefunden");
    }
    
    return status;
  }
  
  /**
   * Kopiert die zwischengespeicherten Daten gemss der bergebenen Teilhierarchie in neue Hierarchieobjekte
   * der ursprnglichen Hierarchie
   * 
   * @param ho das zu kopierende HO in der Teilhierarchie
   * @param srcKB der (Hierarchie-) KB, aus dem kopiert wird
   * @param destKB der (Hierarchie-) KB, in dem kopiert wird
   * @param vater neues Vater-Objekt, unter dem das zu erstellende neue HO angelegt wird
   * @return den Status der Aktion
   */
  public HOMitStatus kopiereHORekursiv(HierarchieObjekt ho, ConfigurationArea srcKB, ConfigurationArea destKB, 
      Object vater)
  {
    return kopiereHORekursiv(ho, srcKB, destKB, vater, true);
  }
  
  /**
   * Kopiert die zwischengespeicherten Daten gemss der bergebenen Teilhierarchie in neue Hierarchieobjekte
   * der ursprnglichen Hierarchie
   * 
   * @param ho das zu kopierende HO in der Teilhierarchie
   * @param srcKB der (Hierarchie-) KB, aus dem kopiert wird
   * @param destKB der (Hierarchie-) KB, in dem kopiert wird
   * @param vater neues Vater-Objekt, unter dem das zu erstellende neue HO angelegt wird
   * @param isWurzel bestimmt, ob die Wurzel oder eine untergeordnetes Objekt der Teilhierarchie kopiert wird
   * @return den Status der Aktion
   */
  protected HOMitStatus kopiereHORekursiv(HierarchieObjekt ho, ConfigurationArea srcKB, ConfigurationArea destKB, 
      Object vater, boolean isWurzel)
  {
    MultiStatus meldung;
    
    if (ho == _hierarchie.getWurzel())
    {  
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
    }
    else
    {
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere untergeordnetes Hierarchieobjekt " + 
          ho + " zu " + vater, null);
    }
    
    ConfigurationArea kb = ho.getSystemObjekt(0).getConfigurationArea();
    if (kb == srcKB)
    {
      kb = destKB;
    }
    
    String[] pid = new String[ho.getSystemObjekte().size()];
    for (int i = 0; i < pid.length; i += 1)
    {
      pid[i] = ho.getSystemObjekt(i).getPid();
    }
    
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    
    HOMitStatus result = _hm.erzeugeNeuesHO(kb, vater, pid, hot, false);
    meldung.merge( result.getStatus() );
    HierarchieObjekt neu = result.getHO();
    
    if (neu != null)
    {
      if (!hot.isSystemObjektVorhanden())
      { 
        meldung.merge(kopiereZwischengespeichertDaten(neu, ho));
      } 
      
      for (HierarchieObjekt kind: _hierarchie.getUntergeordneteObjekte( ho ))
      {
        result = kopiereHORekursiv(kind, srcKB, destKB, neu, false);
        meldung.add( result.getStatus() );
      }
    }
    
    return new HOMitStatus(neu, meldung);
  }
  
}
