/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse der Elemente des Navigators
 * 
 * @author Krause
 *
 */
public abstract class AbstractNavigatorElement implements INavigatorElement
{ 
  
  /**
   * Flag zum Debuggen
   */
  protected static final boolean m_debug = false;
  
  /**
   * mgliche Kategorien des NavigatorElements, welches den
   * Bildarten des NavigatorElements entspricht, die zur Verfgung stehen
   * - falls eine neue Kategorie (Bildart) hinzu kommt, muss der entsprechende Filename
   *   bis zum "_" hier eingetragen werden -
   */
  public static enum KATEGORIE {unbekannt, 
    Wetterstation, KonfigBereich, Inselbus, Unterzentrale, 
    Streckenstation, EinAusKonz, KommRechnerIF, Datenendgeraet, 
    DatenendgeraetDe128, DatenendgeraetDeAxl, DatenendgeraetDeLve, 
    DatenendgeraetDeSys, DatenendgeraetDeUfd, DatenendgeraetDeUfdNW, 
    DatenendgeraetDeVlt, DatenendgeraetDeWww, DatenendgeraetDeWzg,
    Anzeigequerschnitte, Anzeigequerschnitt, Anzeige, 
    Messquerschnitte, Messquerschnitt, Unterspannvers, Fahrspur,
    Ordner, Kalender, Ereignis, Ereignistyp};

    /**
     * mgliche TLS-Kategorien des NavigatorElements, welches den
     * Bildarten des NavigatorElements entspricht, die zur Verfgung stehen
     * - falls eine neue Kategorie (Bildart) hinzu kommt, muss der entsprechende Filename
     *   bis zum "_" hier eingetragen werden -
     */
    public static enum KATEGORIE_TLS {unbekannt, 
      Konfigurationsbereich, VerkehrsInformationsZentrale, VerkehrsRechnerZentrale, UnterZentrale, 
      KommunikationsRechnerInterface, Inselbus, SteuerModul, EinAusgabeKonzentrator, DatenEndgeraet, 
      DatenendgeraetDe128, DatenendgeraetDeAxl, DatenendgeraetDeLve, 
      DatenendgeraetDeSys, DatenendgeraetDeUfd, DatenendgeraetDeUfdNW, 
      DatenendgeraetDeVlt, DatenendgeraetDeWww, DatenendgeraetDeWzg};

  /**
   * Anzahl der maximal mglichen Stati zu den Bildarten (Elementtypen)
   */
  private static final HashMap<KATEGORIE, Integer> ANZAHLSTATI = new HashMap<KATEGORIE, Integer>();
  static
  {
    ANZAHLSTATI.put( KATEGORIE.Datenendgeraet,          6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDe128,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeAxl,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeLve,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeSys,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeUfd,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeUfdNW,   6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeVlt,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeWww,     6 );
    ANZAHLSTATI.put( KATEGORIE.DatenendgeraetDeWzg,     6 );
    ANZAHLSTATI.put( KATEGORIE.EinAusKonz,              6 );
    ANZAHLSTATI.put( KATEGORIE.Inselbus,                6 );
    ANZAHLSTATI.put( KATEGORIE.KommRechnerIF,           6 );
    ANZAHLSTATI.put( KATEGORIE.KonfigBereich,           6 );
    ANZAHLSTATI.put( KATEGORIE.Streckenstation,         6 );
    ANZAHLSTATI.put( KATEGORIE.Unterzentrale,           6 );
    ANZAHLSTATI.put( KATEGORIE.Wetterstation,           6 );
    ANZAHLSTATI.put( KATEGORIE.Anzeigequerschnitt,      6 );
    ANZAHLSTATI.put( KATEGORIE.Anzeigequerschnitte,     6 );
    ANZAHLSTATI.put( KATEGORIE.Ereignis,                5 );
    ANZAHLSTATI.put( KATEGORIE.Anzeige,                 4 );
    ANZAHLSTATI.put( KATEGORIE.Messquerschnitt,         2 );
    ANZAHLSTATI.put( KATEGORIE.Fahrspur,                2 );
    ANZAHLSTATI.put( KATEGORIE.Messquerschnitte,        2 );
    ANZAHLSTATI.put( KATEGORIE.Unterspannvers,          2 );
    ANZAHLSTATI.put( KATEGORIE.Ordner,                  0 );
    ANZAHLSTATI.put( KATEGORIE.Kalender,                0 );
    ANZAHLSTATI.put( KATEGORIE.Ereignistyp,             0 );
    ANZAHLSTATI.put( KATEGORIE.unbekannt,               0 );
  }
  
  /**
   * Ordner in dem sich die Bilder befinden
   */  
  private static final String IMAGEORDNER = "icons/items/";
  
  /**
   * mgliche Bildstati fr die Bildart
   * - enthlt zu den Stati die entsprechenden Endungen der Imagefiles inklusive "_"
   * - maximal mglich sind: "_unb.png", "_IO.png", "_tIO.png", "_tSt.png", "_TeilSt.png", "_St.png"
   * - Sonderstati fr Ereignisse: "_unb.png", "_ungueltig.png", "_zeit.png", "_verkehr.png", "_zeitverkehr.png"
   */
  private final HashMap<STATUS,String> m_imagestatus;
  
  /**
   * Kategorie des NavigatorElements
   * entspricht der Bildart, welche das Bild kennzeichnet 
   */
  private KATEGORIE  m_kategorie;

  /**
   * Flag, ob die Kinder ausgeklappt sind
   */
  private boolean m_ausgeklappt;

  /**
   * Status (unbekannt, Strung, usw)
   */
  private STATUS m_status;

  /**
   * Name, der im Baum angezeigt wird
   */
  private String m_name;

  /**
   * Bildpfad zum Bild, das im Baum angezeigt wird
   */
  private String m_imagePath;
  
  /**
   * Elternteil, das bergeordnet ist (auch Wurzel genannt)
   */
  private Object m_parent;
  
  /**
   * Liste der direkten Kinder (untergeordnete Elemente)
   */
  private List<Object> m_childrenList;

  //--------------------------------------------------------------------- 
  /**
   * Konstruktor fr ein Element des Navigators
   * 
   * @param parent
   *          Elternteil
   * @param name
   *          angezeigter Name
   * @param kategorie
   *          Kategorie des NavigatorElements, entspricht dem Teil des Filenames
   *          bis zum "_"
   */
  protected AbstractNavigatorElement(Object parent, String name, KATEGORIE kategorie)
  {
    this.m_parent = parent;
    this.m_name = name;
    this.m_ausgeklappt = false;
    this.m_childrenList = new ArrayList<Object>();
   
    if ( parent != null )
    {
      ((INavigatorElement) parent).addChild( this );
    }
    
    int anzahlStati = 0;
    if ( kategorie != null )
    {
      this.m_kategorie = kategorie;
      anzahlStati = ANZAHLSTATI.get( m_kategorie );
    }
    
    this.m_imagestatus = new HashMap<STATUS, String>();
    
    //HashMap der Bildstati mit den Endungen der Imagefiles fllen
    if (m_imagestatus.size() == 0 && anzahlStati == 5)
    {
      //Sonderfall bei Ereignissen
      this.m_imagestatus.put( STATUS.unbekannt, "_unb.png" );
      this.m_imagestatus.put( STATUS.ungueltig, "_ungueltig.png" );
      this.m_imagestatus.put( STATUS.zeitlich_gueltig, "_zeit.png" );
      this.m_imagestatus.put( STATUS.verkehrlich_gueltig, "_verkehr.png" );
      this.m_imagestatus.put( STATUS.zeitverkehrlich_gueltig, "_zeitverkehr.png" );
      
    }
    else if ( m_imagestatus.size() == 0 && anzahlStati > 0 )
    {
      this.m_imagestatus.put( STATUS.unbekannt, "_unb.png" );
      this.m_imagestatus.put( STATUS.IO, "_IO.png" );
      if ( anzahlStati >= 4 )
      {
        this.m_imagestatus.put( STATUS.TeilStoerung, "_TeilSt.png" );
        this.m_imagestatus.put( STATUS.Stoerung, "_St.png" );
        
        if ( anzahlStati == 6 )
        {
          this.m_imagestatus.put( STATUS.teilweiseIO, "_tIO.png" );
          this.m_imagestatus.put( STATUS.teilweiseStoerung, "_tSt.png" );
        }
      }
    }

    if (m_imagestatus.size() == 0 && kategorie == null )
    {
      //nderungsstati
      this.m_imagestatus.put( STATUS.unbekannt, "_unb.png" );
      this.m_imagestatus.put( STATUS.nichtAenderbar, "_nAenderbar.png" );
      this.m_imagestatus.put( STATUS.teilweiseAenderbar, "_tAenderbar.png" );
      this.m_imagestatus.put( STATUS.aenderbar, "_aenderbar.png" );    
    }
    
    // Status "unbekannt" vorbelegen
    if ( name != null )
      setStatus( STATUS.unbekannt );
    
  }

  /**
  * Liefert das zugeordnete Objekte
  */
  abstract public Object getObjekt();
   
  /**
  * Liefert die zugeordneten SystemObjekte
  */
  abstract public List<SystemObject> getObjekte();
  
  @Override
  public Object[] getChildren()
  {
    return m_childrenList.toArray();
  }

  @Override
  public String getImagePath()
  {
    return m_imagePath;
  }
  
  @Override
  public Object getParent()
  {
    return m_parent;
  }
   
  @Override
  public STATUS getStatus()
  {
    return m_status;
  }

  @Override
  public String getName()
  {
    return m_name;
  }
  
  @Override
  public boolean hasObjekt(SystemObject objekt)
  {
    boolean enthalten = false;
    for (Iterator<SystemObject> iterator = this.getObjekte().iterator(); iterator.hasNext();)
    {
      SystemObject systemobjekt = (SystemObject) iterator.next();
      if ( systemobjekt.equals( objekt ) )
      {
        enthalten = true;
        break;
      }   
    }
    
    return enthalten;
      

  }
  
  @Override
  public boolean isAusgeklappt()
  {
    return m_ausgeklappt;
  }

  @Override
  public void setAusgeklappt( boolean b )
  {
    m_ausgeklappt = b;
  }

  @Override
  public void setChildren( ArrayList<Object> children )
  {
    m_childrenList = children;
  }

  @Override
  public void addChild( INavigatorElement child )
  {
    m_childrenList.add( child );
  }

  @Override
  /**
   * Setzt den Bildpfad.
   * Falls nur ein Dateiname (ohne .) bergeben wird,
   * wird dieser zum kompletten Bildpfad ergnzt.
   * 
   * @param bildPfad  kompletter Bildpfad oder Dateiname
   */
  public void setImagePath( String bildPfad )
  {
    if ( !bildPfad.contains(".") )
      bildPfad = IMAGEORDNER + bildPfad + ".png";
    m_imagePath = bildPfad;
  }
  
  public void setName( String name )
  {
    m_name = name;
  }
  
  @Override
  public void setParent( INavigatorElement parent )
  {
    m_parent = parent;
  }


  @Override
  public boolean hasChildren()
  {
    // Rckgabe, ob das Element Kinder hat
    return m_childrenList == null ? false : m_childrenList.size() > 0;
  }

  @Override
  public boolean hasParent()
  {
    // Rckgabe, ob das Element einen Elternteil hat
    return m_parent == null ? false : true;
  }
  
  /**
   * Setzt den Status und das entsprechende Bild
   * sowie den Status des Elternteils und die Aktualisierung des Teilbaums
   * 
   * @param status    
   *          Status des NavigatorElementes
   * @param IMAGEORDNER 
   *          Ordner in dem sich die Images befinden
   * @param m_kategorie 
   *          Kategorie des NavigatorElements also Bildart, welche verwendet werden soll         
   */
  @Override
  public boolean setStatus( STATUS status )
  {

    //Bild zum Status entsprechend setzen
    String imagefile = null;
    
    if ( status != null && m_imagestatus.size()>0 )
      imagefile = m_imagestatus.get( status );
    
    if ( imagefile != null && m_kategorie != null )
      imagefile = IMAGEORDNER + m_kategorie.name() + imagefile;
    else if ( m_kategorie != null )
      imagefile = IMAGEORDNER + m_kategorie.name() + ".png";
    else if ( imagefile != null && m_imagePath != null )
      imagefile = m_imagePath.substring(0, m_imagePath.indexOf(".")) + imagefile;
    else
      imagefile = null;
    
    if ( m_debug )
      System.out.println("Imagefile: " + imagefile);

    if (imagefile != null && !imagefile.equals( "" ))
      setImagePath( imagefile );
        
    //Status setzen, Bild zum Elternteil neu setzen und TeilBaum aktualisieren
    if ( status != null )
    {
      if (m_debug)
        System.out.println("Setze neuen Status: " + status);
      
      STATUS statusalt = getStatus();
      m_status = status;

      //Status und Bild zum Elternteil neu setzen
      setParentStatus(status, statusalt);
      
      //TODO TeilBaum aktualisieren
      
    }
    
    return true;
    
  }

  /**
   * Status und Bild zum Elternteil neu setzen
   * 
   * @param status      neuer Status des Element
   * @param statusalt   alter Status des Element
   */
  private void setParentStatus(STATUS status, STATUS statusalt)
  {
    if (m_debug)
    {
      System.out.println("Werte fr Autostatus: ");
      System.out.println("Statusalt: " + statusalt + " Statusneu: " + status);
    }
    INavigatorElement parent = (INavigatorElement) getParent();
    
    if ( parent != null && status != null && !status.equals( statusalt ) )
    {
      //Beim Start unntige Statusermittlungen des Elternteils unterlassen
//        if (!( statusalt == null && status.equals( STATUS.unbekannt ) && parent.getStatus().equals( STATUS.unbekannt ) ))
//        {
        if (m_debug)
          System.out.println( "AutoStatus setzen bei " + parent.getName() );
        parent.setAutoStatus();
//        }
    }
  }

  @Override
  public boolean setAutoStatus()
  {
    if ( m_kategorie == null || ANZAHLSTATI.get( m_kategorie ) == 0 )
      return false;
    
    STATUS status = null;
    STATUS statusneu = null;

    Object[] children = getChildren();
    for (int i = 0; i < children.length; i++)
    {
      INavigatorElement child = (INavigatorElement) children[i];
      statusneu = child.getStatus();

      if (m_debug)
        System.out.println( "Aktueller Status: " + status + " Status von Child: " + statusneu );

      if ( statusneu == null )
      {
        continue;
      }

      if ( status == null )
      {
        status = statusneu;
        continue;
      }

      switch (status)
        {
        case unbekannt:
        {
          switch (statusneu)
            {
            case IO:
            case teilweiseIO:
              if ( ANZAHLSTATI.get( m_kategorie ) == 6 )
                status = STATUS.teilweiseIO;
              else
                status = STATUS.IO;
              break;
            case Stoerung:
            case teilweiseStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) == 6 )
                status = STATUS.teilweiseStoerung;
              else
                status = STATUS.Stoerung;                
              break;
            case TeilStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) >= 4 )
                status = STATUS.TeilStoerung;
              else
                status = STATUS.unbekannt;
              break;
            }
          break;
        }
        case IO:
        {
          switch (statusneu)
            {
            case unbekannt:
            case teilweiseIO:
              if ( ANZAHLSTATI.get( m_kategorie ) == 6 )
                status = STATUS.teilweiseIO;
              else
                status = STATUS.IO;
              break;
            case Stoerung:
            case teilweiseStoerung:
            case TeilStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) >= 4 )
                status = STATUS.TeilStoerung;
              else
                status = STATUS.unbekannt;
              break;
            }
          break;
        }
        case Stoerung:
        {
          switch (statusneu)
            {
            case unbekannt:
            case teilweiseStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) == 6 )
                status = STATUS.teilweiseStoerung;
              else
                status = STATUS.Stoerung;  
              break;
            case IO:
            case teilweiseIO:
            case TeilStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) >= 4 )
                status = STATUS.TeilStoerung;
              else
                status = STATUS.unbekannt;
              break;
            }
          break;
        }
        case teilweiseIO:
        {
          switch (statusneu)
            {
            case Stoerung:
            case teilweiseStoerung:
            case TeilStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) >= 4 )
                status = STATUS.TeilStoerung;
              else
                status = STATUS.unbekannt;
              break;
            }
          break;
        }
        case teilweiseStoerung:
        {
          switch (statusneu)
            {
            case IO:
            case teilweiseIO:
            case TeilStoerung:
              if ( ANZAHLSTATI.get( m_kategorie ) >= 4 )
                status = STATUS.TeilStoerung;
              else
                status = STATUS.unbekannt;
              break;
            }
          break;
        }
        }//switch status
    }

    if ( status == null )
      return false;
    else
    {
      setStatus( status );
      return true;
    }
  }

  /**
   * Liefert die nderbarkeit
   * 
   * @return Status
   */
  public boolean isAenderbar()
  {
//    System.out.println("isAenderbar: " + m_status + " Size Objekte: " +  getObjekte().size());
    
    boolean aenderbar = false;
    if (m_status == STATUS.aenderbar)
      aenderbar = true;
    else if (m_status == STATUS.teilweiseAenderbar && getObjekte().size() > 1)
      aenderbar = true;
    
    return aenderbar;
  }

  /**
   * Liefert die Verschiebbarkeit
   * 
   * @return Status
   */
  public boolean isVerschiebbar()
  {
    boolean verschieben = false;
    if (m_status == STATUS.aenderbar)
      verschieben = true;
    else if (m_status == STATUS.teilweiseAenderbar)
      verschieben = true;
    
    return verschieben;
  }
  
  /**
   * Liefert die Kategorie (auch Bildart)
   */
  public KATEGORIE getKategorie()
  {
    return m_kategorie;
  }
      
  /**
   * Liefert den Tooltip
   * 
   * @return Tooltip
   */
  public String getToolTip()
  {
    return null;
  }

  /**
   * Liefert den Suffix
   * 
   * @return Suffix
   */
  public String getSuffix()
  {
    return null;
  }
  
  //------------------------------------------------------------
  // Methoden fr nderungen im DatenModell
  //------------------------------------------------------------
  
  /**
   * Kind zum Element hinzufgen
   */ 
  public void add(INavigatorElement toAdd)
  {
    if (m_debug)
      System.out.println("Element " + toAdd.getName() + " hinzufgen!");
    
    m_childrenList.add( toAdd );
    toAdd.setParent( this );

  }
  
  /**
   * Kind vom Element entfernen
   */
  public void remove(INavigatorElement toRemove)
  {
    if (m_debug)
      System.out.println("Element " + toRemove.getName() + " entfernen!");
    m_childrenList.remove( toRemove );
    
  } 
  
}
