/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkaufd;

import java.util.Arrays;
import java.util.Collection;
import java.util.Vector;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.konfigass.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Prfungen von Attributwerten fr den TKA Umfelddaten
 * 
 * @author Sans
 *
 */
public class PruefungenUFD implements IPruefungen
{
	/**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

  /**
   * Die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	private IHierarchie _hierarchie;
	
	 /**
   * Bestimmt die Hierarchie, die fr die Prfungen benutzt werden kann
   * 
   * @return die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	/**
   * Setzt die Hierarchie, die fr die Prfungen benutzt werden kann
   * @param hierachie die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
	
	/**
	* Prft die Daten, die fr ein Hierarchieobjekt bernommen werden sollen
	* 
	* @param ho das Hierarchieobjekt, fr das Daten bernommen werden sollen
	* @param daten die zu bernehmenden Daten
	* @return eine Instanz vom Typ IStatus, die den Status der Prfung und alle 
	* Informationen enthlt
	*/
	public IStatus pruefeHierarchieObjekt(HierarchieObjekt ho, HODaten daten)
	{
		// Ergebnisstatus dieser Prfung, der in der Meldungen-Sicht ausgegeben wird
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
				"Prfe Hierarchieobjekt " + ho, null);
	  		
		// Alle Systemobjekte des aktuellen HO werden berprft
		for (SystemObject so: ho.getSystemObjekte())
  	{
			 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
			 if (soDaten != null)
			 {
				 // Wenn das aktuelle HO mehr als ein Systemobjekt enthlt, wird in der
				 // Statusmeldung eine Ebene eingezogen
				 MultiStatus objektStatus;
				 if (ho.getSystemObjekte().size() > 1)
				 {
					 objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
							 "Prfe Systemobjekt " + so.getPid() + " vom Typ " + so.getType().getPid(), null);
				 }
				 else
				 {
					 objektStatus = result;
				 }
				 
				 // Allgemeine Prfungen: PruefungenTools#pruefePid
				 ConfigDataModel dataModel = getHierarchie().getDataModel();
				 PruefungenTools pruefungenTools = new PruefungenTools(dataModel);
				 objektStatus.add(pruefungenTools.pruefePid(soDaten.getPid(), so));	
				 				 				 
				 if (so.isOfType("typ.umfeldDatenSensor"))
				 {
					 // Spezifische Prfung fr den Typ typ.umfeldDatenSensor
					 // pruefeZuordnungSensor prft, ob der Sensor nur maximale einer 
					 // Messstelle zugeordnet ist
					 objektStatus.add(pruefeZuordnungSensor(ho, so, soDaten));
				 }
				 
				 if (objektStatus != result)
				 {
					 // Korrekter Zusammenbau der Statusmeldung, auch wenn mehrere
					 // Systemobjekte vorhanden sind
					 result.add(objektStatus);
				 }
			 }	
  	}
		
		return result;
	}
	
	/**
   * Prft die Daten einer Attributgruppe, die fr eine Systemobjekt eines
   * Hierarchieobjekts bernommen werden sollen
   * 
   * @param ho das Hierarchieobhekt, zu dem das bergebene Systemobjekt gehrt
   * @param so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
   * @return eine Instanz vom Typ IStatus, die aussagt, ob es zulssig ist, die 
   * bergebenen ATG-Daten fr das Systemobjekt zu bernehmen.
   */
	public IStatus pruefeDaten(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten)
	{
		// Ergebnisstatus dieser Prfung, der in der Meldungen-Sicht ausgegeben wird
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID,	Status.OK, 
  			"Prfe Daten der ATG " + atgInfo.getAtg(), null);
		
		 if (so.isOfType("typ.umfeldDatenMessStelle"))
		 {
			  // Fr den Typ typ.umfeldDatenMessStelle sind nur allgemeine
			  // Prfungen definiert 
				if ("atg.punktKoordinaten".equals(atgInfo.getAtg()))
				{
					result.merge(PruefungenTools.pruefeDatenDefinition(atgInfo, daten));
				}
		 }
		 else if (so.isOfType("typ.umfeldDatenSensor"))
		 {
			 if ("atg.umfeldDatenSensor".equals(atgInfo.getAtg())) 
			 {
				 // Allgemeine und spezifische Prfungen fr die ATG 
				 // atg.punktKoordinaten des Typs typ.umfeldDatenSensor
				 pruefeDatenAtgUmfeldDatenSensor(ho, so, atgInfo, daten, result);
			 }
			 else if ("atg.punktKoordinaten".equals(atgInfo.getAtg()))
			 {
				 // Allgemeine Prfungen fr die ATG atg.punktKoordinaten
				 // des Typs typ.umfeldDatenSensor
				 result.merge(PruefungenTools.pruefeDatenDefinition(atgInfo, daten));
			 }
		 }
		
		return result;
	}
   
	/**
	 * Es wird geprft, ob jeder Messstelle maximal ein Sensor zugeordnet ist
	 *  
	 * @param ho das Hierarchieobhekt, zu dem das bergebene Systemobjekt gehrt
   * @param so das zu prfende Systemobjekt
	 * @param soDaten die Daten, die zu dem Systemobjekt geprft werden sollen
   * @return Status, der das Ergebnis der Prfung wiedergibt
	 */
	private IStatus pruefeZuordnungSensor(HierarchieObjekt ho, SystemObject so, SystemObjektDaten soDaten)
  {
   	Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
  	MultiStatus result = null;
   	
  	if (vater instanceof HierarchieObjekt && 
  			((HierarchieObjekt)vater).getHierarchieObjektTyp().equals(HOT_UFD.UmfeldDatenMessStelle))
  	{
  		SystemObject ms = ((HierarchieObjekt)vater).getSystemObjekt(0);
  		NonMutableSet menge = ((ConfigurationObject)ms).getNonMutableSet("UmfeldDatenSensoren");
  		if (menge != null)
  		{
  			if (menge.getElementsInModifiableVersion().contains(so))
  			{
  				ConfigDataModel dataModel = _hierarchie.getDataModel();
    	  	    	  	
  				for (ConfigurationArea kb: dataModel.getAllConfigurationAreas().values())
  				{
  					SystemObjectType[] typSpez = new SystemObjectType[] { dataModel.getType("typ.umfeldDatenMessStelle") };
  					Collection<SystemObject> msListe = new Vector<SystemObject>();
  					msListe.addAll(kb.getObjects(Arrays.asList(typSpez), ObjectTimeSpecification.valid()));
  					
  					for (SystemObject lauf: kb.getNewObjects())
  					{
							if (lauf.isOfType("typ.umfeldDatenMessStelle"))
							{
								msListe.add(lauf);
							}
  					}
  					
  					for (SystemObject lauf: msListe)
  					{
  						if (!lauf.equals(ms))
  						{	
	  						ConfigurationObject ko = (ConfigurationObject)lauf;
	  						
	  						if (!(ko.getNotValidSince() >= kb.getModifiableVersion()))
	  		        // Objekt wird in zuknftiger Version nicht ungltig
			          {
									NonMutableSet set = ((ConfigurationObject)lauf).getNonMutableSet("UmfeldDatenSensoren");
									if (set != null)
									{
										if (set.getElementsInModifiableVersion().contains(so))
										{
											if (result == null)
											{
												result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
														"Der UFD-Sensor " + so.getPid() + " darf nicht mehreren Messstellen zugeordnet sein", null);
											}
											result.add( new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
													"Der UFD-Sensor " + so.getPid() + " ist zustzlich der Messstelle " + lauf.getPid() + " zugeordnet"));
										}
									}
			          }
  						}	
  					}
  				}
  			}
  		}
  		else
  		{
  			_debug.warning("Menge UmfeldDatenSensorem am Objekt " + ms.getPid() + " nicht vorhanden");
  		}
  	}
  	else
  	{
  		throw new IllegalStateException("bergeordnetes Objekt von " + ho + " vom Typ " +
  				HierarchieObjekt.class + " vom HOT " + HOT_UFD.UmfeldDatenMessStelle + " erwartet");
   	}
  	  	
  	if (result == null)
  	{
  		return new Status(IStatus.OK, Activator.PLUGIN_ID, 
  				"Prfen Zuordnung UFD-Sensor zur Messstelle: OK");
		}
  	else
  	{
  		return result;
  	}
  }
	
	/**
	 * Fr die bergebenen ATG-Daten werden pro Attribut folgende Prfungen ausgefhrt:<br>
	 * 1. Korrekte Definition<br>
	 * 2. Bei Textfeldern: Zulssige Lnge<br>
	 * 3. Pflichtfeldeigenschaft<br>
	 * Folgende spezifischen Prfungen werden ausgefhrt:<br>
	 * 1. Vorgnger und Nachfolger drfen nicht auf die Umfelddatenmessstelle verweisen, an denen der Sensor eingetragen wurde<br>
   * 2. Vorgnger und Nachfolger drfen nicht auf diesselbe Umfelddatenmessstelle verweisen<br>
   * 3. Der Ersatzsensor darf nicht der Sensor selbst sein<br>
	 * 4.	Der Ersatzsensor muss vom selben Typ wie der Sensor sein<br>
	 * 
	 * @param ho das Hierarchieobhekt, zu dem das bergebene Systemobjekt gehrt
   * @param so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
	 * @param status Instanz vom Typ IStatus, der fr jedes Attribut der bergebenen
	 * Attributgruppe eine Meldung mit dem Ergebnis der Prfung hinzugefgt wird.
	 */
	private void pruefeDatenAtgUmfeldDatenSensor(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	SystemObject vorgaenger = null;
  	SystemObject nachfolger = null;
  		  	
  	Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
  	Assert.isTrue(vater instanceof HierarchieObjekt);
  	Assert.isTrue(HOT_UFD.UmfeldDatenMessStelle.equals(((HierarchieObjekt)vater).getHierarchieObjektTyp()));
  	
  	SystemObject sensor = ((HierarchieObjekt)vater).getSystemObjekt(0);
  	
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
  		
  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
  			if ("Vorgnger".equals(attribut))
    		{
    			vorgaenger = daten.getItem(attribut).asReferenceValue().getSystemObject(); 
    			
    			if (vorgaenger == null || vorgaenger != sensor)
    	  	{
    	  		meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfe Zuordnung Vorgnger: OK"));
    	  	}
    	  	else
    	  	{
    	  		meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
    				"Vorgnger darf nicht die bergeordnetet Messstelle sein"));
    	  	}
    		}
    		if ("Nachfolger".equals(attribut))
    		{
    			nachfolger = daten.getItem(attribut).asReferenceValue().getSystemObject();
    			
    			if (nachfolger == null || nachfolger != sensor)
    	  	{
    	  		meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfe Zuordnung Nachfolger: OK"));
    	  	}
    	  	else
    	  	{
    	  		meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
    				"Nachfolger darf nicht die bergeordnetet Messstelle sein"));
    	  	}
    		}
    		if ("ErsatzSensor".equals(attribut))
    		{
    			SystemObject ersatzSensor = daten.getItem(attribut).asReferenceValue().getSystemObject();
    			
    			if (ersatzSensor == null || ersatzSensor != ho.getSystemObjekt(0))
    	  	{
    	  		meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfe Zuordnung Ersatzsensor: OK"));
    	  	}
    	  	else
    	  	{
    	  		meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
    				"Ersatzsensor darf nicht der aktuelle Sensor sein"));
    	  	}
    	  	
    	  	if (ersatzSensor == null || ersatzSensor.getType().equals(ho.getSystemObjekt(0).getType()))
    	  	{
    	  		meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfe Typ Ersatzsensor: OK"));
    	  	}
    	  	else
    	  	{
    	  		meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
    				"Ersatzsensor darf nicht von anderem Typ wie der aktuelle Sensor sein"));
    	  	}
    		}
    		if ("Typ".equals(attribut))
    		{
    			String typ = daten.getItem(attribut).asTextValue().getText();
    			if (typ != null && "HauptSensor".equals(typ))
    			{
    				boolean fehler = false;
    				ConfigDataModel dataModel = _hierarchie.getDataModel();
    				ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
    				AttributeGroup atg = dataModel.getAttributeGroup(atgInfo.getAtg());
    				
    				for (HierarchieObjekt kind: _hierarchie.getUntergeordneteObjekte(vater))
    				{
    					if (kind != ho && kind.getHierarchieObjektTyp().equals(ho.getHierarchieObjektTyp()))
    					{
    						SystemObject kindSensor = kind.getSystemObjekt(0);
    						Data data = ((ConfigSystemObject)kindSensor).getConfigurationData(atg, lookup);
    						
    						if (data != null)
    						{	
    							String kindTyp = data.getItem(attribut).asTextValue().getText();
    							if (kindTyp != null && kindTyp.equals(typ))
    							{	
		    						meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Typ '" + typ + 
	  								"' wird auch benutzt von " + ho.getHierarchieObjektTyp() + "-Sensor " + kind));
		    						fehler = true;
    							}	
    						}	
    					}
    				}
    				if (!fehler)
	    			{
	    				meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
	    				"Prfe Eindeutigkeit Attribut Typ '"+ typ + "': OK"));
	    			}
    			}
    		}
			}
  		
  		status.add(meldung);
  	}	
  	
  	if (vorgaenger == null || nachfolger == null || vorgaenger != nachfolger)
  	{
  		status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
			"Prfe Ungleichheit Vorgnger und Nachfolger: OK"));
  	}
  	else
  	{
  		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
			"Vorgnger und Nachfolger drfen nicht gleich sein"));
  	}
  }
	 
}
  