/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkaufd;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Instanzen von UFDSUnterTyp sind Untertypen vom HOT_UFD UFDS
 * 
 * @author Sans
 *
 */
public class UFDSUnterTyp  implements IHierarchieObjektTyp
{
	/**
	 * Name des Systemobjekt-Obertyps fr Umfelddatensensoren
	 */
	public final static String UFDS_OBER_TYP = "typ.umfeldDatenSensor";
	
	/**
	 * Prefix fr die Systemobjekt-Untertypen fr Umfelddatensensoren
	 */
	public final static String UFDS_TYP_PREFIX = "typ.ufds";
	
  /**
   * Name des HOT - wird in der Navigationsdarstellung verwendet
   */
  private String _name;
  
  /**
   * Die Systemobjekt-Typen, die von diesem HOT verwendet werden
   */
  private List<String> _typen = new Vector<String>();
  
  /**
   * Die Typinfos zu den Systemobjekttypen
   */
  private Map<String, TypInfo> _typInfos = new HashMap<String, TypInfo>();
  
    
  
  /**
   * Erzeugen eines neuen HOT_UFD auf Basis des HOT UFDS.<br>
   * Es werden alle Infos des bergeordneten HOT UFDS fr den Systemobjekttyp UFDS_OBER_TYP 
   * in die neu zu erzeugende Instanz fr den Sytemobjekttyp <code>typ</code> bernommen.<br>
   * Die Erzeugung eines HOT auf diese Art mit mehr als einem Systemobjekttyp ist nicht
   * mglich
   * 
   * @param typ Systemobjekttyp des neuen HOT
   * 
   */
  public UFDSUnterTyp(String typ)
  {
    TypInfo typInfo = HOT_UFD.UFDS.getTypInfo(UFDS_OBER_TYP);
    List<String> atgs = typInfo.getAtgs();
    AtgInfo atgInfo[] = new AtgInfo[atgs.size()];
    for (int i = 0; i < atgInfo.length; i += 1)
    {
    	atgInfo[i] = typInfo.getAtgInfo(atgs.get(i)); 
    }
    List<String> mengen = typInfo.getMengen();
    MengeInfo mengeInfo[] = new MengeInfo[mengen.size()];
    for (int i = 0; i < mengeInfo.length; i += 1)
    {
    	mengeInfo[i] = typInfo.getMengeInfo(mengen.get(i)); 
    }
    
    _name = bestimmeName(typ);
    _typen.add(typ);
    _typInfos.put(typ, new TypInfo(typ, atgInfo, mengeInfo));
  }
  
  /**
   * Bestimm den Name des HOT aus dem Namen eiens Systemobjekttyps<br>
   * Wenn der Name des Systemobjekttyps das Pattern typ.ufdsXXX erfllt, wird der
   * HOT-Name UFDS_XXX gebildet, ansonsten wird der Name des Systemobjekttyps, 
   * umgewandelt in Grossbuchstaben, zurckgegeben
   * 
   * @param typ der Name des zugrundeliegenden Systemobjekttyps
   * @return den Namen des HOT
   */
  private String bestimmeName(String typ)
	{
		String result = null;
		
		if (typ.startsWith(UFDS_TYP_PREFIX)) // Pattern typ.ufdsXXX
		{
			int len = UFDS_TYP_PREFIX.length();
			if (typ.length() > len)
			{
				result = "UFDS_" + typ.substring(len);
			}
		}
		
		if (result == null || "".equals(result)) 
		{
			result = typ.toUpperCase();
		}
		
		return result;
	}	
  
  /**
   * Text-Reprsentation fr diesen HOT
   * 
   * @return den Namen des HOT - wird in der Navigationsdarstellung verwendet
   */
  @Override
  public String toString()
  {
    return getName();
  }
    
  /**
	 * @see IHierarchieObjektTyp#getName()
	 */
  public String getName()
  {
    return _name;
  }
  
  /**
	 * @see IHierarchieObjektTyp#getSymbolName()
	 */
  public String getSymbolName()
  {
  	return "UFDS";
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypen()
	 */
  public List<String> getTypen()
  {
    return Collections.unmodifiableList(_typen); 
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypInfo(String)
	 */
  public TypInfo getTypInfo(String typ)
  {
  	return _typInfos.get(typ);
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypInfo(SystemObjectType)
	 */
  public TypInfo getTypInfo(SystemObjectType sot)
  {
  	TypInfo result = getTypInfo(sot.getPid());
  	
  	if (result == null && !sot.isBaseType())
  	{
  		for (SystemObjectType superTyp: sot.getSuperTypes())
  		{
  			if (superTyp.isConfigurating())
  			{
  				 result = getTypInfo(superTyp);
  				 if (result != null)
  				 {
  					 break;
  				 }
  			}
  		}
  	}
  	
  	return result;
  }

  /**
	 * @see IHierarchieObjektTyp#isSystemObjektVorhanden()
	 */
  public boolean isSystemObjektVorhanden()
  {
    return false;
  }
  
  public boolean isSystemObjektNichtAenderbar()
  {
  	return isSystemObjektVorhanden();
  }

}
